unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, OpenGL, Textures, ExtCtrls, StdCtrls;

const
  ArtistCount = 3;
  Artists : array[0..ArtistCount] of String = ('Madonna.jpg','Avril Lavigne.jpg','Christina Aguilera.jpg','Britney Spears.jpg');
  HeaderWidth = 426;
  HeaderHeight = 104;

type
  TfrmMain = class(TForm)
    tOpenGLDraw : TTimer;
    bLoadClick: TButton;
    bLoadThread: TButton;
    procedure FormCreate (Sender : TObject);
    procedure tOpenGLDrawTimer (Sender : TObject);
    procedure SetUpArtistImages;
    procedure LoadArtistTextures;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure bLoadClickClick(Sender: TObject);
    procedure bLoadThreadClick(Sender: TObject);
  private
  public
  end;

var
  frmMain : TfrmMain;
  DC : HDC;
  RC : HGLRC;
  ArtistImages : array[0..ArtistCount] of GLUInt;
  W, H : GLFloat;
  ThreadHandle : Integer;

implementation

{$R *.dfm}

procedure glBindTexture (Target : GLEnum; Texture : GLUInt); stdcall; external OpenGL32;
procedure glGenTextures (N : GLSizeI; var Textures : GLUInt); stdcall; external OpenGL32;


procedure TfrmMain.FormCreate ( Sender : TObject
                              );


  procedure SetUpPixelFormat ( DC : HDC
                             );
  const
    PFD : TPIXELFORMATDESCRIPTOR = (
          nSize : SizeOf (TPIXELFORMATDESCRIPTOR); nVersion : 1;
          dwFlags : PFD_SUPPORT_OPENGL or PFD_DRAW_TO_WINDOW or PFD_DOUBLEBUFFER;
          iPixelType : PFD_TYPE_RGBA;
          cColorBits : 24; cRedBits : 0; cRedShift : 0; cGreenBits : 0;
          cGreenShift : 0; cBlueBits : 0; cBlueShift : 0; cAlphaBits : 0;
          cAlphaShift : 0; cAccumBits : 0; cAccumRedBits : 0; cAccumGreenBits : 0;
          cAccumBlueBits : 0; cAccumAlphaBits : 0; cDepthBits : 16; cStencilBits : 0;
          cAuxBuffers : 0; iLayerType : PFD_MAIN_PLANE; bReserved : 0; dwLayerMask : 0;
          dwVisibleMask : 0; dwDamageMask : 0);
  var
    PixelFormat : Integer;
  begin
    PixelFormat := ChoosePixelFormat (DC,@PFD);
    if (PixelFormat = 0) then
    begin
      Exit;
    end;

    if (SetPixelFormat (DC,PixelFormat,@PFD) <> True) then
    begin
      Exit;
    end;
  end;

  procedure InitialiseOpenGL;
  begin
    DC := GetDC (frmMain.Handle);
    SetupPixelFormat (DC);
    RC := wglCreateContext (DC);
    wglMakeCurrent (DC,RC);

    glClearColor (0,0,0,0);
    glShadeModel (GL_SMOOTH);
    glBlendFunc (GL_SRC_ALPHA,GL_ONE);
    glHint (GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
  end;

begin
  Width := 600;
  Height := 600;

  InitialiseOpenGL;
  tOpenGLDraw.Enabled := True;

  W := (HeaderWidth / Screen.Width);
  H := (HeaderHeight / Screen.Height);

  glEnable (GL_TEXTURE_2D);
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity;
  gluPerspective (45,frmMain.Width / frmMain.Height,1,100);
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity;
  glTranslate (0,0,-1.5);
end;


procedure TfrmMain.tOpenGLDrawTimer ( Sender : TObject
                                    );
var
  Loop : Integer;
  Y : GLFloat;
begin
  glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  Y := 0.45;
  for Loop := 0 to ArtistCount do
  begin
    glBindTexture (GL_TEXTURE_2D,ArtistImages[Loop]);
    glColor3f (1,1,1);

    glBegin (GL_QUADS);
      glTexCoord2f (0,0);
      glVertex3f (0 - W,Y - H,0);
      glTexCoord2f (1,0);
      glVertex3f (0 + W,Y - H,0);
      glTexCoord2f (1,1);
      glVertex3f (0 + W,Y + H,0);
      glTexCoord2f (0,1);
      glVertex3f (0 - W,Y + H,0);
    glEnd;
    Y := Y - 0.3;
  end;

  SwapBuffers (wglGetCurrentDC);
end;


procedure TfrmMain.SetUpArtistImages;
var
  Loop : Integer;
begin
  for Loop := 0 to ArtistCount do
  begin
    LoadTexture (Artists[Loop],ArtistImages[Loop],False);
  end;
end;


procedure TfrmMain.LoadArtistTextures;
var
  Loop : Integer;
begin
  for Loop := 0 to ArtistCount do
  begin
    LoadTexture (Artists[Loop],ArtistImages[Loop],False);
  end;
  EndThread (0);
end;


procedure TfrmMain.bLoadClickClick ( Sender : TObject
                                   );
begin
  SetUpArtistImages;
end;


procedure TfrmMain.bLoadThreadClick ( Sender : TObject
                                    );
var
  ThreadID : LongWord;
begin
  ThreadHandle := BeginThread (Nil,0,Addr (TfrmMain.LoadArtistTextures),Nil,0,ThreadID);
end;


procedure TfrmMain.FormClose ( Sender : TObject;
                               var Action : TCloseAction
                             );
begin
  CloseHandle (ThreadHandle);
end;


end.
